import os

# Define the folder structure and file content
TEMPLATE_STRUCTURE = {
    "app": {
        "__init__.py": """from flask import Flask
from app.config import Config
from app.extensions import db, jwt, cors
from app.routes import register_blueprints

def create_app():
    app = Flask(__name__)
    app.config.from_object(Config)
    db.init_app(app)
    jwt.init_app(app)
    cors.init_app(app)
    register_blueprints(app)
    return app
""",
        "config.py": """import os

class Config:
    SECRET_KEY = os.getenv("SECRET_KEY", "your_secret_key")
    SQLALCHEMY_DATABASE_URI = os.getenv("DATABASE_URL", "sqlite:///app.db")
    SQLALCHEMY_TRACK_MODIFICATIONS = False
""",
        "extensions.py": """from flask_sqlalchemy import SQLAlchemy
from flask_jwt_extended import JWTManager
from flask_cors import CORS

db = SQLAlchemy()
jwt = JWTManager()
cors = CORS()
""",
        "models/__init__.py": "",
        "models/user.py": """from app.extensions import db

class User(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(80), unique=True, nullable=False)
    password_hash = db.Column(db.String(255), nullable=False)
""",
        "routes/__init__.py": """from flask import Blueprint
from app.routes.auth import auth_bp
from app.routes.user import user_bp

def register_blueprints(app):
    app.register_blueprint(auth_bp)
    app.register_blueprint(user_bp)
""",
        "routes/auth.py": """from flask import Blueprint, request, jsonify
from app.services.auth_service import AuthService

auth_bp = Blueprint('auth', __name__)

@auth_bp.route('/register', methods=['POST'])
def register():
    data = request.json
    return AuthService.register(data)

@auth_bp.route('/login', methods=['POST'])
def login():
    data = request.json
    return AuthService.login(data)
""",
        "routes/user.py": """from flask import Blueprint, jsonify

user_bp = Blueprint('user', __name__)

@user_bp.route('/profile', methods=['GET'])
def profile():
    return jsonify({"message": "User profile endpoint"})
""",
        "middleware/__init__.py": "",
        "middleware/auth_middleware.py": """from flask import request, jsonify
from flask_jwt_extended import verify_jwt_in_request, get_jwt_identity

def auth_required(fn):
    def wrapper(*args, **kwargs):
        try:
            verify_jwt_in_request()
            return fn(*args, **kwargs)
        except Exception as e:
            return jsonify({"error": str(e)}), 401
    return wrapper
""",
        "services/__init__.py": "",
        "services/auth_service.py": """from flask import jsonify
from app.models.user import User
from app.extensions import db
from werkzeug.security import generate_password_hash, check_password_hash

class AuthService:
    @staticmethod
    def register(data):
        username = data.get('username')
        password = data.get('password')
        if not username or not password:
            return jsonify({"error": "Missing username or password"}), 400
        
        hashed_password = generate_password_hash(password)
        new_user = User(username=username, password_hash=hashed_password)
        db.session.add(new_user)
        db.session.commit()
        return jsonify({"message": "User registered successfully"}), 201

    @staticmethod
    def login(data):
        username = data.get('username')
        password = data.get('password')
        user = User.query.filter_by(username=username).first()
        
        if user and check_password_hash(user.password_hash, password):
            return jsonify({"message": "Login successful"}), 200
        return jsonify({"error": "Invalid credentials"}), 401
""",
        "utils/__init__.py": "",
        "utils/validators.py": """def validate_email(email):
    return '@' in email and '.' in email
""",
        "utils/response.py": """from flask import jsonify

def success_response(message, data=None):
    return jsonify({"message": message, "data": data or {}}), 200

def error_response(message, status_code=400):
    return jsonify({"error": message}), status_code
"""
    },
    "migrations": {},
    "instance": {
        "config.py": """# Instance-specific configuration
DEBUG = True
"""
    },
    "tests": {
        "__init__.py": "",
        "test_auth.py": """import unittest
from app import create_app

class AuthTestCase(unittest.TestCase):
    def setUp(self):
        self.app = create_app()
        self.client = self.app.test_client()

    def test_register(self):
        response = self.client.post('/register', json={"username": "testuser", "password": "testpass"})
        self.assertEqual(response.status_code, 201)
"""
    }
}

def create_project_structure():
    base_dir = os.getcwd()
    for folder, files in TEMPLATE_STRUCTURE.items():
        folder_path = os.path.join(base_dir, folder)
        os.makedirs(folder_path, exist_ok=True)
        for file_name, file_content in files.items():
            file_path = os.path.join(folder_path, file_name)
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            if not os.path.exists(file_path):  # Avoid overwriting existing files
                with open(file_path, "w") as f:
                    f.write(file_content)

    print("✅ Flask backend structure generated successfully with pre-filled code!")

def main():
    create_project_structure()

if __name__ == "__main__":
    main()
